==> dvlp-16.1.3.tgz: package/electron-entry.js <==
// src/electron-host/electron-entry.js
import { dirname, join } from "node:path";
import {
  error,
  filePathToUrlPathname,
  getElectronWorkerData,
  interceptInProcess
} from "dvlp/internal";
import { fileURLToPath, pathToFileURL } from "node:url";

// src/utils/regexp.js
function escapeRegExp(string) {
  return string.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
}

// src/electron-host/electron-entry.js
import { syncBuiltinESMExports } from "node:module";

// src/utils/base64Url.js
function toBase64Url(string, isBase64 = false) {
  const base64 = isBase64 ? string : Buffer.from(string, "utf-8").toString("base64");
  return base64.replace(/\+/g, "-").replace(/\//g, "_").replace(/=+$/, "");
}

// src/electron-host/electron-entry.js
import workerThreads from "node:worker_threads";
var RE_DATA_URL = /^data:text\/html;([^,]+,)?/;
var __dirname = dirname(fileURLToPath(import.meta.url));
var workerPath = join(__dirname, "./electron-worker.js");
var cwd = pathToFileURL(process.cwd()).href;
var reFileProtocol = new RegExp(
  `(?<=(href|src)=["|'])(${escapeRegExp(cwd)})`,
  "g"
);
async function bootstrapElectron() {
  const electronWorkerData = getElectronWorkerData();
  const { BrowserWindow } = await import("electron");
  interceptInProcess(electronWorkerData);
  try {
    const WebContents = process._linkedBinding(
      "electron_browser_web_contents"
    ).WebContents;
    WebContents.prototype.loadFile = new Proxy(WebContents.prototype.loadFile, {
      apply(target, ctx, args) {
        const filePath = (
          /** @type { string } */
          args[0]
        );
        const url = new URL(
          filePathToUrlPathname(filePath),
          electronWorkerData.hostOrigin
        );
        return WebContents.prototype.loadURL.call(ctx, url.href);
      }
    });
    WebContents.prototype.loadURL = new Proxy(WebContents.prototype.loadURL, {
      apply(target, ctx, args) {
        let url = (
          /** @type { string } */
          args[0]
        );
        if (url.startsWith("file://") || url.startsWith(electronWorkerData.origin)) {
          const incomingUrl = new URL(url);
          url = new URL(
            incomingUrl.pathname + incomingUrl.search,
            electronWorkerData.hostOrigin
          ).href;
        } else if (RE_DATA_URL.test(url)) {
          const [match, encoding] = (
            /** @type { RegExpExecArray } */
            RE_DATA_URL.exec(url)
          );
          const encodedMarkup = url.replace(match, "");
          const decodedMarkup = encoding === "base64," ? Buffer.from(encodedMarkup, "base64").toString("utf8") : decodeURIComponent(encodedMarkup);
          const markup = toBase64Url(
            // Remove protocol from any element file:// URLs
            decodedMarkup.replaceAll(reFileProtocol, "")
          );
          const argOptions = args[1];
          if (argOptions && "baseURLForDataURL" in argOptions) {
            delete argOptions.baseURLForDataURL;
          }
          url = new URL(`?dvlpdata=${markup}`, electronWorkerData.hostOrigin).href;
        }
        args[0] = url;
        const [preloadPath] = ctx._getPreloadPaths?.() ?? [];
        if (preloadPath) {
          electronWorkerData.postMessage({
            type: "watch",
            filePath: preloadPath,
            mode: "read"
          });
        }
        return Reflect.apply(target, ctx, args);
      }
    });
  } catch (err) {
    console.log(err);
  }
  workerThreads.Worker = new Proxy(workerThreads.Worker, {
    construct(target, args, newTarget) {
      const [filePath, options] = args;
      const { port1, port2 } = new workerThreads.MessageChannel();
      port1.unref();
      port1.on(
        "message",
        /** @param { ElectronProcessMessage} msg */
        (msg) => {
          if (msg.type === "listening") {
            electronWorkerData.origin ??= msg.origin;
            electronWorkerData.postMessage(msg);
          }
        }
      );
      electronWorkerData.postMessage({ type: "watch", filePath, mode: "read" });
      options.workerData ??= {};
      options.transferList ??= [];
      args[0] = workerPath;
      options.workerData.dvlp = {
        hostOrigin: electronWorkerData.hostOrigin,
        main: filePath,
        messagePort: port2,
        serializedMocks: electronWorkerData.serializedMocks
      };
      options.transferList.push(port2);
      return Reflect.construct(target, args, newTarget);
    }
  });
  syncBuiltinESMExports();
  process.on("message", (msg) => {
    if (msg === "close") {
      BrowserWindow.getAllWindows().forEach((window) => window.close());
    }
  });
  process.on("uncaughtException", error);
  process.on("unhandledRejection", error);
  try {
    await import(pathToFileURL(electronWorkerData.main).href);
    electronWorkerData.postMessage({ type: "started" });
  } catch (err) {
    error(err);
  }
}
export {
  bootstrapElectron
};

==> dvlp-16.2.0.tgz: package/electron-entry.js <==
// src/electron-host/electron-entry.js
import { dirname, join } from "node:path";
import {
  error,
  filePathToUrlPathname,
  getElectronWorkerData,
  interceptInProcess
} from "dvlp/internal";
import { fileURLToPath, pathToFileURL } from "node:url";

// src/utils/regexp.js
function escapeRegExp(string) {
  return string.replace(/[.*+?^${}()|[\]\\]/g, "\\$&");
}

// src/electron-host/electron-entry.js
import { syncBuiltinESMExports } from "node:module";

// src/utils/base64Url.js
function toBase64Url(string, isBase64 = false) {
  const base64 = isBase64 ? string : Buffer.from(string, "utf-8").toString("base64");
  return base64.replace(/\+/g, "-").replace(/\//g, "_").replace(/=+$/, "");
}

// src/electron-host/electron-entry.js
import workerThreads from "node:worker_threads";
var RE_DATA_URL = /^data:text\/html;([^,]+,)?/;
var RE_HTTP_URL = /^https?:\/\//;
var __dirname = dirname(fileURLToPath(import.meta.url));
var workerPath = join(__dirname, "./electron-worker.js");
var cwd = pathToFileURL(process.cwd()).href;
var reFileProtocol = new RegExp(
  `(?<=(href|src)=["|'])(${escapeRegExp(cwd)})`,
  "g"
);
async function bootstrapElectron() {
  const electronWorkerData = getElectronWorkerData();
  const { BrowserWindow } = await import("electron");
  interceptInProcess(electronWorkerData);
  try {
    const WebContents = process._linkedBinding(
      "electron_browser_web_contents"
    ).WebContents;
    WebContents.prototype.loadFile = new Proxy(WebContents.prototype.loadFile, {
      apply(target, ctx, args) {
        const filePath = (
          /** @type { string } */
          args[0]
        );
        const url = new URL(
          filePathToUrlPathname(filePath),
          electronWorkerData.hostOrigin
        );
        return WebContents.prototype.loadURL.call(ctx, url.href);
      }
    });
    WebContents.prototype.loadURL = new Proxy(WebContents.prototype.loadURL, {
      apply(target, ctx, args) {
        let url = (
          /** @type { string } */
          args[0]
        );
        if (url.startsWith("file://") || RE_HTTP_URL.test(url)) {
          const incomingUrl = new URL(url);
          url = new URL(
            incomingUrl.pathname + incomingUrl.search,
            electronWorkerData.hostOrigin
          ).href;
        } else if (RE_DATA_URL.test(url)) {
          const [match, encoding] = (
            /** @type { RegExpExecArray } */
            RE_DATA_URL.exec(url)
          );
          const encodedMarkup = url.replace(match, "");
          const decodedMarkup = encoding === "base64," ? Buffer.from(encodedMarkup, "base64").toString("utf8") : decodeURIComponent(encodedMarkup);
          const markup = toBase64Url(
            // Remove protocol from any element file:// URLs
            decodedMarkup.replaceAll(reFileProtocol, "")
          );
          const argOptions = args[1];
          if (argOptions && "baseURLForDataURL" in argOptions) {
            delete argOptions.baseURLForDataURL;
          }
          url = new URL(`?dvlpdata=${markup}`, electronWorkerData.hostOrigin).href;
        }
        args[0] = url;
        const [preloadPath] = ctx._getPreloadPaths?.() ?? [];
        if (preloadPath) {
          electronWorkerData.postMessage({
            type: "watch",
            filePath: preloadPath,
            mode: "read"
          });
        }
        return Reflect.apply(target, ctx, args);
      }
    });
  } catch (err) {
    console.log(err);
  }
  workerThreads.Worker = new Proxy(workerThreads.Worker, {
    construct(target, args, newTarget) {
      const [filePath, options] = args;
      const { port1, port2 } = new workerThreads.MessageChannel();
      port1.unref();
      port1.on(
        "message",
        /** @param { ElectronProcessMessage} msg */
        (msg) => {
          if (msg.type === "listening") {
            electronWorkerData.postMessage(msg);
          }
        }
      );
      electronWorkerData.postMessage({ type: "watch", filePath, mode: "read" });
      options.workerData ??= {};
      options.transferList ??= [];
      args[0] = workerPath;
      options.workerData.dvlp = {
        hostOrigin: electronWorkerData.hostOrigin,
        main: filePath,
        messagePort: port2,
        serializedMocks: electronWorkerData.serializedMocks
      };
      options.transferList.push(port2);
      return Reflect.construct(target, args, newTarget);
    }
  });
  syncBuiltinESMExports();
  process.on("message", (msg) => {
    if (msg === "close") {
      BrowserWindow.getAllWindows().forEach((window) => window.close());
    }
  });
  process.on("uncaughtException", error);
  process.on("unhandledRejection", error);
  try {
    await import(pathToFileURL(electronWorkerData.main).href);
    electronWorkerData.postMessage({ type: "started" });
  } catch (err) {
    error(err);
  }
}
export {
  bootstrapElectron
};
